using Microsoft.EntityFrameworkCore;

namespace EFCore6Test;

public class SampleContext : DbContext
{
    public SampleContext(DbContextOptions options) : base(options)
    {
    }
    
    public DbSet<Book> Books { get; set; }
    public DbSet<Album> Albums { get; set; }
    public DbSet<User> Users { get; set; }
    public DbSet<Group> Groups { get; set; }
    public DbSet<Product> Products { get; set; }
    public DbSet<Team> Teams { get; set; }

    protected override void OnModelCreating(ModelBuilder modelBuilder)
    {
        modelBuilder.Entity<Book>(entity =>
        {
            entity.HasOne(d => d.Album)
                .WithMany(p => p.Books)
                .HasForeignKey(d => d.AlbumId);
            entity.HasOne(d => d.User)
                .WithMany(p => p.Books)
                .HasForeignKey(d => d.UserId);
            entity.HasOne(d => d.ParentBook)
                .WithMany(p => p.InverseParentBook)
                .HasForeignKey(d => d.ParentBookId);
            entity.HasOne(d => d.PrivateGroup)
                .WithMany(p => p.BookPrivateGroups)
                .HasForeignKey(d => d.PrivateGroupId);
        });
        modelBuilder.Entity<Album>(entity =>
        {
            entity.HasOne(d => d.OwnerUser)
                .WithMany(p => p.AlbumOwnerUsers)
                .HasForeignKey(d => d.OwnerUserId);
            entity.HasOne(d => d.Book)
                .WithMany(p => p.Albums)
                .HasForeignKey(d => d.BookId);
        });
        modelBuilder.Entity<User>(entity =>
        {
            entity.HasOne(d => d.CreatedByProduct)
                .WithMany(p => p.Users)
                .HasForeignKey(d => d.CreatedByProductId);
            entity.HasOne(d => d.Book)
                .WithMany(p => p.Users)
                .HasForeignKey(d => d.BookId);
            entity.HasOne(d => d.ReaderGroup)
                .WithMany(p => p.UserReaderGroups)
                .HasForeignKey(d => d.ReaderGroupId);
        });
        modelBuilder.Entity<Product>(entity =>
        {
            entity.HasOne(d => d.ReadGroup)
                .WithMany(p => p.ProductReadGroups)
                .HasForeignKey(d => d.ReadGroupId);
        });
        modelBuilder.Entity<Group>(entity =>
        {
            entity.HasOne(d => d.OwnerAlbum)
                .WithMany(p => p.Groups)
                .HasForeignKey(d => d.OwnerAlbumId)
                .OnDelete(DeleteBehavior.SetNull);
            entity.HasOne(d => d.OwnerUser)
                .WithMany(p => p.Groups)
                .HasForeignKey(d => d.OwnerUserId);
            entity.HasOne(d => d.OwnerProduct)
                .WithMany(p => p.Groups)
                .HasForeignKey(d => d.OwnerProductId);
        });
        modelBuilder.Entity<Team>(entity =>
        {
            entity.HasOne(d => d.Album)
                .WithMany(p => p.Teams)
                .HasForeignKey(d => d.AlbumId);
        });
    }
}

public class Book
{
    public int Id { get; set; }
    public int? ParentBookId { get; set; }
    public int? PrivateGroupId { get; set; }
    public int? AlbumId { get; set; }
    public int? UserId { get; set; }

    public Album Album { get; set; }
    public User User { get; set; }
    public Book ParentBook { get; set; }
    public Group PrivateGroup { get; set; }
    public ICollection<Album> Albums { get; set; }
    public ICollection<User> Users { get; set; }
    public ICollection<Book> InverseParentBook { get; set; }
}

public class Album
{
    public int Id { get; set; }
    public int? BookId { get; set; }
    public int? OwnerUserId { get; set; }

    public User OwnerUser { get; set; }
    public Book Book { get; set; }
    public ICollection<Book> Books { get; set; }
    public ICollection<Group> Groups { get; set; }
    public ICollection<Team> Teams { get; set; }
}

public class User
{
    public int Id { get; set; }
    public int? BookId { get; set; }
    public int? ReaderGroupId { get; set; }
    public int? CreatedByProductId { get; set; }

    public Product CreatedByProduct { get; set; }
    public Book Book { get; set; }
    public Group ReaderGroup { get; set; }
    public ICollection<Album> AlbumOwnerUsers { get; set; }
    public ICollection<Book> Books { get; set; }
    public ICollection<Group> Groups { get; set; }
}

public class Group
{
    public int Id { get; set; }
    public int? OwnerAlbumId { get; set; }
    public int? OwnerUserId { get; set; }
    public int? OwnerProductId { get; set; }

    public Album OwnerAlbum { get; set; }
    public User OwnerUser { get; set; }
    public Product OwnerProduct { get; set; }
    public ICollection<User> UserReaderGroups { get; set; }
    public ICollection<Book> BookPrivateGroups { get; set; }
    public ICollection<Product> ProductReadGroups { get; set; }
}

public class Product
{
    public int Id { get; set; }
    public int? ReadGroupId { get; set; }

    public Group ReadGroup { get; set; }
    public ICollection<User> Users { get; set; }
    public ICollection<Group> Groups { get; set; }
}

public class Team
{
    public int Id { get; set; }
    public int AlbumId { get; set; }
    public Album Album { get; set; }
}